@@ -2,10 +2,12 @@ from .support import TickLiveData, MinLiveData, TouchlineData
 from .support import TDLiveDataError, TDHistoricDataError
 from .TD_live import LiveClient
 from .TD_hist import HistoricalREST
+from .TD_chain import OptionChain
 from threading import Thread
 from typing import Union, Callable, Any
 from collections import defaultdict
 
+
 from datetime import datetime
 from dateutil.relativedelta import relativedelta
 import time
@@ -198,6 +200,15 @@ class TD:
         # return_completed = True  # Uncomment and remove arg to disallow user choice
         return self.historical_datasource.bhavcopy(segment, date, return_completed)
 
+    def get_gainers(self , segment , topn , df_style = True ):
+        topn = 10 if topn is None else topn
+        return self.historical_datasource.get_gainers_losers(segment, topn , gainers = True , df_style = df_style )
+
+    def get_losers(self , segment , topn , df_style = True ):
+        topn = 10 if topn is None else topn
+        return self.historical_datasource.get_gainers_losers(segment , topn , gainers = False , df_style = df_style )
+
+
     @staticmethod
     def get_req_id_list(req_id: Union[int, list], len_contracts: int) -> list:
         if type(req_id) == list:
@@ -212,35 +223,53 @@ class TD:
             raise TDLiveDataError("Invalid req_id datatype...")
 
     def start_live_data(self, resolved_contracts, req_id=None):  # TODO: Prevent reuse of req_ids
+        symbols_to_call = []
         if req_id is not None:
             req_ids = self.get_req_id_list(req_id, len(resolved_contracts))
         else:
             req_ids = self.get_req_id_list(self.default_market_data_id, len(resolved_contracts))
             self.default_market_data_id = self.default_market_data_id + len(resolved_contracts)
-        symbols_to_call = []
-        for j in range(0, len(req_ids)):
-            resolved_contract = resolved_contracts[j].upper()
-            self.touchline_data[req_ids[j]] = TouchlineData()
+        for i , req_id in enumerate(req_ids):
+            resolved_contract = resolved_contracts[i].upper()
+            if resolved_contract in self.symbol_mkt_id_map.keys():
+                self.touchline_data[req_id] = self.touchline_data[list(self.symbol_mkt_id_map[resolved_contract])[0]]
+            else:
+                self.touchline_data[req_id] = TouchlineData()
             if self.live_websocket.subscription_type == 'tick':
-                if req_ids[j] not in self.live_data.keys() or type(self.live_data[req_ids[j]]) != TickLiveData:
-                    self.live_data[req_ids[j]] = TickLiveData(resolved_contract)
+                if resolved_contract in self.symbol_mkt_id_map.keys():
+                    self.live_data[req_id] = self.live_data[list(self.symbol_mkt_id_map[resolved_contract])[0]]
+                elif req_id not in self.live_data.keys() or type(self.live_data[req_id]) != TickLiveData:
+                    self.live_data[req_id] = TickLiveData(resolved_contract)
             elif '+' in self.live_websocket.subscription_type:
-                if req_ids[j] not in self.live_data.keys() or type(self.live_data[req_ids[j]]) != TickLiveData:
-                    self.live_data[req_ids[j]] = TickLiveData(resolved_contract)
-                if req_ids[j] not in self.min_live_data.keys() or type(self.min_live_data[req_ids[j]]) != MinLiveData:
-                    self.min_live_data[req_ids[j]] = MinLiveData(resolved_contract)
+                if resolved_contract in self.symbol_mkt_id_map.keys():
+                    self.live_data[req_id] = self.live_data[list(self.symbol_mkt_id_map[resolved_contract])[0]]
+                elif req_id not in self.live_data.keys() or type(self.live_data[req_id]) != TickLiveData:
+                    self.live_data[req_id] = TickLiveData(resolved_contract)
+                if resolved_contract in self.symbol_mkt_id_map.keys():
+                    self.min_live_data[req_id] = self.min_live_data[list(self.symbol_mkt_id_map[resolved_contract])[0]]
+                elif req_id not in self.min_live_data.keys() or type(self.min_live_data[req_id]) != MinLiveData:
+                    self.min_live_data[req_id] = MinLiveData(resolved_contract)
             elif 'min' in self.live_websocket.subscription_type:
-                if req_ids[j] not in self.live_data.keys() or type(self.live_data[req_ids[j]]) != MinLiveData:
-                    self.live_data[req_ids[j]] = MinLiveData(resolved_contract)
+                if resolved_contract in self.symbol_mkt_id_map.keys():
+                    self.live_data[req_id] = self.live_data[list(self.symbol_mkt_id_map[resolved_contract])[0]]
+                elif req_id not in self.live_data.keys() or type(self.live_data[req_id]) != MinLiveData:
+                    self.live_data[req_id] = MinLiveData(resolved_contract)
             if resolved_contract not in self.symbol_mkt_id_map.keys():
                 symbols_to_call.append(resolved_contract)
-            self.symbol_mkt_id_map[resolved_contract].update({req_ids[j]})
+            self.symbol_mkt_id_map[resolved_contract].update({req_id})
         if len(symbols_to_call) > 0:
             self.live_websocket.send(f'{{"method": "addsymbol", "symbols": {json.dumps(symbols_to_call)}}}')
         return req_ids
 
     def stop_live_data(self, contracts):  # Clearing objects is done after server confirmation
-        self.live_websocket.send(f'{{"method": "removesymbol", "symbols": {json.dumps(contracts)}}}')
+        remove_symbols = []
+        for sym in contracts:
+            if len(self.symbol_mkt_id_map[sym]) > 1:
+                self.symbol_mkt_id_map[sym].remove(list(self.symbol_mkt_id_map[sym])[-1])
+            else:
+                remove_symbols.append(sym)
+                self.symbol_mkt_id_map.pop(sym , None)
+        self.live_websocket.send(f'{{"method": "removesymbol", "symbols": {json.dumps(remove_symbols)}}}')
 
     def trade_callback(self, func: Callable[[TickLiveData], Any]):
         self.logger.info(f"Defining {func} as trade_callback...")
@@ -268,3 +297,16 @@ class TD:
 
     # def get_touchline(self):
     #     self.live_websocket.send(f'{{"method": "touchline"}}')
+
+    def start_option_chain( self , symbol , expiry , chain_length = None , bid_ask = False , market_open_post_hours = False ):
+        chain_length = 10 if chain_length is None else chain_length  # setting default value for chain length
+        future_price = self.get_n_historical_bars(f'{symbol}-I' )[0]['c']
+        try:
+            chain = OptionChain(self , symbol, expiry , chain_length , future_price , bid_ask  , market_open_post_hours )
+        except Exception as e:
+            print(f'Please check symbol: {symbol} and its expiry: {expiry.date()}')
+            exit()
+        symb_ids = self.start_live_data(chain.option_symbols)
+        option_thread = Thread(target=chain.update_chain, args=( self , symb_ids ), daemon=True)
+        option_thread.start()
+        return chain
