@@ -23,7 +23,7 @@ class OptionChain:
                  chain_length ,
                  future_price,
                  bid_ask ,
-                 market_close ):
+                 market_open_post_hours ):
         self.TD_OBJ = TD_OBJ
         self.login_id = TD_OBJ.login_id
         self.password = TD_OBJ.password
@@ -36,7 +36,7 @@ class OptionChain:
         self.chain_dataframe =  self.init_dataframe()
         self.chain_status = False
         self.bid_ask = bid_ask
-        self.market_close = market_close
+        self.market_open_post_hours = market_open_post_hours
 
     def get_strike_step(self ):
         expiry = self.expiry.strftime('%Y%m%d')
@@ -51,14 +51,15 @@ class OptionChain:
     def get_option_symbols(self ):
         expiry = self.expiry.strftime('%y%m%d')
         future_atm = round(self.future_price / self.strike_step ) * self.strike_step
-        start_strike = future_atm - self.strike_step * int(self.chain_length / 2 )
-        end_strike = future_atm + self.strike_step * int( self.chain_length / 2)
+        start_strike = int(future_atm - self.strike_step * int(self.chain_length / 2 ) )
+        end_strike = int(future_atm + self.strike_step * int( self.chain_length / 2) )
         symbols = list(map(lambda x: f'{self.symbol}{expiry}{x}CE', range( start_strike , end_strike  , self.strike_step )))
         symbols.extend(list(map(lambda x: f'{self.symbol}{expiry}{x}PE', range( start_strike , end_strike  , self.strike_step ))))
         return symbols
 
     def init_dataframe(self):
-        df = pd.DataFrame(columns=['symbols' , 'strike' , 'type' , 'ltp' , 'ltt' , 'oi' , 'oi_change_perc' , 'bid' , 'bid_qty' , 'ask' , 'ask_qty'])
+        df = pd.DataFrame(columns=['symbols' , 'strike' , 'type' , 'ltp' , 'ltt' ,'volume' , 'oi' , 'prev_oi' , 'oi_change' ,
+                                    'oi_change_perc' , 'bid' , 'bid_qty' , 'ask' , 'ask_qty'])
         df.symbols = self.option_symbols
         df.strike = df.symbols.apply(lambda x: str(re.findall('\d+' , x )[0])[6:])
         df.type = df.symbols.apply(lambda x: re.findall('\D+' , x )[-1] )
@@ -76,10 +77,12 @@ class OptionChain:
                 oi_change_perc = round((td_obj.touchline_data[symb_id].oi - td_obj.touchline_data[symb_id].prev_oi)*100 / (td_obj.touchline_data[symb_id].prev_oi) , 2 )
             except ZeroDivisionError:
                 oi_change_perc = 0
-            self.chain_dataframe.loc[td_obj.touchline_data[symb_id].symbol , ['ltp' , 'ltt'  , 'oi' , 'oi_change_perc' , 'bid' , 'bid_qty' , 'ask' , 'ask_qty' ]] = [
-                td_obj.touchline_data[symb_id].ltp, td_obj.touchline_data[symb_id].timestamp ,td_obj.touchline_data[symb_id].oi, oi_change_perc  ,
-                td_obj.touchline_data[symb_id].best_bid_price,td_obj.touchline_data[symb_id].best_bid_qty, td_obj.touchline_data[symb_id].best_ask_price,
-                td_obj.touchline_data[symb_id].best_ask_qty]
+            self.chain_dataframe.loc[td_obj.touchline_data[symb_id].symbol , ['ltp' , 'ltt'  , 'volume' ,  'oi' , 'prev_oi', 'oi_change' ,
+                                                                                'oi_change_perc','bid' , 'bid_qty' , 'ask' , 'ask_qty' ]] = [
+                td_obj.touchline_data[symb_id].ltp, td_obj.touchline_data[symb_id].timestamp , td_obj.touchline_data[symb_id].ttq ,
+                td_obj.touchline_data[symb_id].oi,td_obj.touchline_data[symb_id].prev_oi , td_obj.touchline_data[symb_id].oi - td_obj.touchline_data[symb_id].prev_oi ,
+                oi_change_perc , td_obj.touchline_data[symb_id].best_bid_price,td_obj.touchline_data[symb_id].best_bid_qty,
+                td_obj.touchline_data[symb_id].best_ask_price,td_obj.touchline_data[symb_id].best_ask_qty]
 
         while self.chain_status:
             for symb_id in symb_ids:
@@ -89,16 +92,18 @@ class OptionChain:
                         ask , ask_qnty = td_obj.live_data[symb_id].best_ask_price, td_obj.live_data[symb_id].best_ask_qty
                     except Exception as e:
                         bid, bid_qnty, ask, ask_qnty = 0, 0, 0, 0
-                    self.chain_dataframe.loc[td_obj.live_data[symb_id].symbol , ['ltt'  , 'oi' , 'oi_change_perc' , 'bid' , 'bid_qty' , 'ask' , 'ask_qty' ]] = [
-                        td_obj.live_data[symb_id].timestamp ,td_obj.live_data[symb_id].oi, round(td_obj.live_data[symb_id].to_dict()['oi_change_perc'] ,2 ) ,
-                        bid, bid_qnty, ask, ask_qnty]
+                    self.chain_dataframe.loc[td_obj.live_data[symb_id].symbol , ['ltt' , 'volume' ,  'oi' ,'prev_oi' , 'oi_change' ,
+                                                                                'oi_change_perc' ,'bid' , 'bid_qty' , 'ask' , 'ask_qty' ]] = [
+                        td_obj.live_data[symb_id].timestamp ,td_obj.live_data[symb_id].ttq , td_obj.live_data[symb_id].oi,
+                        td_obj.live_data[symb_id].prev_day_oi , td_obj.live_data[symb_id].oi_change ,
+                        round(td_obj.live_data[symb_id].to_dict()['oi_change_perc'] ,2 ) ,bid, bid_qnty, ask, ask_qnty]
                     if 'ltp' in td_obj.live_data[symb_id].__dict__.keys():
                         self.chain_dataframe.loc[td_obj.live_data[symb_id].symbol , ['ltp'] ] = td_obj.live_data[symb_id].ltp
                     else:
                         self.chain_dataframe.loc[td_obj.live_data[symb_id].symbol , ['ltp'] ] = td_obj.live_data[symb_id].close
                     prev_tick_obj[symb_id] = deepcopy(td_obj.live_data[symb_id])
             time.sleep(0.005)
-            if self.market_close and dt.now().time() > OptionChain.close_time :
+            if not(self.market_open_post_hours) and dt.now().time() > OptionChain.close_time :
                 break
         # print('exited chain thread')
 
